#!/usr/bin/python
import pygraphviz as pgv

class Node():
    def __init__(self,token,value):
        self._token = token
        self._value = value
        self._children = []

    def token(self):
        return self._token

    def value(self):
        return self._value

    def children(self):
        return self._children

class Trie():
    def __init__(self):
        self._root = []

    def __iter__(self):
        return self.inorder(self._root)

    def inorder(self,t):
        for l in t:
            yield l.value()
            for x in self.inorder(l.children()):
                yield x

    def _add(self,trie, key, content):
        # is the key a leaf
        k = key.pop(0)
        if key == []:
            node = Node(k,content)
            trie.append(node)
        else:
            for ch in trie:
                if ch.token() == k:
                    self._add(ch.children(), key, content)

    def add(self,key, content):
        self._add(self._root, key, content)

    def _graph(self, trie, G):
        for l in trie:
            G.add_node(l.token())
            for ch in l.children():
                G.add_edge(l.token(),ch.token())
                self._graph(l.children(), G)

    def graph(self):
        G = pgv.AGraph(directed=True)
        G.add_node("sitemap")
        for ch in self._root:
            G.add_edge("sitemap",ch.token())
        self._graph(self._root, G)
#        G.layout('dot')
#        G.draw('g.png')
#        print G.string()

    def _menu(self, trie, lang, page, css, subdir):
        html = "<ul%s>\n" % css
        for l in trie:
            sel = ''
            p = l.value().page(lang)
            if p == page:
                sel = ' class="selected"'
            if p != None:
                html += '<li%s><a href="%s%s">%s</a>\n' \
                    % (sel,subdir,l.value().link(),p.menu())
            else:
                html += '<li%s><a href="%s%s.en" hreflang="en">%s</a>*\n' \
                    % (sel,subdir,l.value().link(), l.value().page('en').menu())
            if l.children():
                html += self._menu(l.children(), lang, page, "", subdir)
        html += "</ul>\n"
        return html

    def menu(self,lang,page,cssclass,subdir):
        css = ''
        if cssclass:
            css = ' class="'+cssclass+'"'
        return self._menu(self._root, lang, page, css, subdir)
